﻿"""
/***************************************************************************
Name	        : menu_from_project plugin
Description          : Build layers shortcut menu based on QGis project
Date                 :  10/11/2011 
copyright            : (C) 2011 by AEAG
email                : xavier.culos@eau-adour-garonne.fr 
***************************************************************************/

/***************************************************************************
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
***************************************************************************/

todo:alerte si le projet est configuré <Paths><Absolute>False 

W:\dev_tools\OSGeo4W\bin>lrelease "W:\projets\QGis plugins\menu_from_project\i18n\menu_from_project_fr.ts"

"""
# Import the PyQt and QGIS libraries
import os
from qgis.core import *

from PyQt4 import QtWebKit
from PyQt4.QtCore import * 
from PyQt4.QtGui import *
from PyQt4 import QtXml
from ui_browser import Ui_browser

from menu_conf_dlg import menu_conf_dlg

# Initialize Qt resources from file resources.py
import resources

try:
    _fromUtf8 = QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

try:
    _toUtf8 = QString.toUtf8
except AttributeError:
    _toUtf8 = lambda s: s

try:
    _toAscii = QString.toAscii
except AttributeError:
    _toAscii = lambda s: s

class menu_from_project: 

    def __init__(self, iface):
        self.path = QFileInfo(os.path.realpath(__file__)).path()
        self.iface = iface
        self.toolBar = None
        
        # old single project var
        #self.filePath = None
        # new multi projects var
        self.projects = []
        self.menubarActions = []
        self.canvas = self.iface.mapCanvas()
        self.read()       
        
        # récup langue par défaut
        locale = QSettings().value("locale/userLocale").toString()
        self.myLocale = locale[0:2]
        # exploiter le bon dictionnaire
        localePath = self.path+"/i18n/menu_from_project_" + self.myLocale + ".qm"
        # initialiser le traducteur
        if QFileInfo(localePath).exists():
            self.translator = QTranslator()
            self.translator.load(localePath)
            if qVersion() > '4.3.3':
                QCoreApplication.installTranslator(self.translator)

    def store(self):
        s = QSettings()
        #if self.filePath != None:
        #    s.setValue("menu_from_project/projectFilePath", self.filePath)
        s.remove("menu_from_project/projectFilePath")

        index = 0
        s.beginWriteArray("menu_from_project/projects")
        for project  in self.projects:
            s.setArrayIndex(index)
            s.setValue("file", project["file"])
            s.setValue("name", project["name"])
            index = index + 1
        s.endArray()


    def read(self):
        s = QSettings()

        #try:
        # old single project conf         
        filePath = s.value("menu_from_project/projectFilePath", None).toString()
        if (filePath != ""):
            title = filePath.split('/')[-1]
            title = title.split('.')[0]
            self.projects.append({"file":filePath, "name":title})
            self.store()
        else:
            size = s.beginReadArray("menu_from_project/projects")
            for i in range(size):
                s.setArrayIndex(i)
                file = ((s.value("file").toString()))
                name =((s.value("name").toString()))
                if file != "":
                    self.projects.append({"file":file, "name":(name)})
            s.endArray()
        
        #except:
        #    pass

    def addMenuItem(self, filename, node, menu):
        if node == None:
            return
            
        element = node.toElement()

        # if legendlayer tag
        if element.tagName() == "legendlayer":
            legendlayerfile = node.firstChild().firstChild()
            layerId = legendlayerfile.toElement().attribute("layerid")
            action = QAction(element.attribute("name"), self.iface.mainWindow())
            #action.setData(QVariant(layerId))

            #if element.attribute("name") == "Circonscription administrative AG":
            #    QMessageBox.information(None, "Cancel", str(action))
                
            menu.addAction(action) 
            QObject.connect(action, SIGNAL("triggered()"),  lambda _filename=filename,_who=layerId: self.do_aeag_menu(_filename, _who))

            #if element.attribute("name") == "Circonscription administrative AG":
            #    QMessageBox.information(None, "Cancel", str("ok"))

            node = node.nextSibling()
            if (node != None):
                # ! recursion
                self.addMenuItem(filename, node, menu)
                
        # if legendgroup tag
        if element.tagName() == "legendgroup":
            name = element.attribute("name")
            if name == "-":
                menu.addSeparator()
                node = node.nextSibling()
                if (node != None):
                    # ! recursion
                    self.addMenuItem(filename, node, menu)
            else:
                # construire sous-menu
                sousmenu = menu.addMenu('&'+element.attribute("name"))
                childNode = node.firstChild()
                
                #  ! recursion
                self.addMenuItem(filename, childNode, sousmenu)
                
                nextNode = node.nextSibling()
                if (nextNode != None):
                    # ! recursion
                    self.addMenuItem(filename, nextNode, menu)
        
    def addMenu(self, name, filename, domdoc):
        # main project menu
        menuBar = self.iface.editMenu().parentWidget()
        projectMenu = QMenu('&'+name, menuBar) 
        projectAction = menuBar.addMenu(projectMenu)
        self.menubarActions.append(projectAction);

        # build menu on legend schema
        legends = domdoc.elementsByTagName("legend")
        if (legends.length() > 0):
            node = legends.item(0)
            if (node != None):
                node = node.firstChild()
                self.addMenuItem(filename, node, projectMenu)

    def initMenus(self):
        menuBar = self.iface.editMenu().parentWidget()
        for action in self.menubarActions:
            #QMessageBox.information(None, "Cancel", str("del"))
            menuBar.removeAction(action)
            del(action)
            
        self.menubarActions = []

        idx = 0
        for project in self.projects:
            #QMessageBox.information(None, "Cancel", str("open " + _toAscii(project["name"])))
            #try:
            xml = file(unicode(project["file"])).read()
            doc = QtXml.QDomDocument()
            doc.setContent(xml)
            
            self.addMenu(project["name"], project["file"], doc)
            #except:
            #    pass
            
            idx = idx + 1
        
    def initGui(self):          
        

        self.act_aeag_menu_config = QAction(QApplication.translate("menu_from_project", "Projects configuration", None, QApplication.UnicodeUTF8)+"...", self.iface.mainWindow())
        self.iface.addPluginToMenu(QApplication.translate("menu_from_project", "&Layers menu from project", None, QApplication.UnicodeUTF8), self.act_aeag_menu_config)
        # Add actions to the toolbar
        QObject.connect(self.act_aeag_menu_config, SIGNAL("triggered()"),  self.do_aeag_menu_config)

        self.act_aeag_menu_help = QAction(QApplication.translate("menu_from_project", "Help", None, QApplication.UnicodeUTF8)+"...", self.iface.mainWindow())
        self.iface.addPluginToMenu(QApplication.translate("menu_from_project", "&Layers menu from project", None, QApplication.UnicodeUTF8), self.act_aeag_menu_help)
        QObject.connect(self.act_aeag_menu_help, SIGNAL("triggered()"),  self.do_help)
        
        # build menu
        self.initMenus()


    def unload(self):
        menuBar = self.iface.editMenu().parentWidget()
        for action in self.menubarActions:
            menuBar.removeAction(action)

        self.iface.removePluginMenu(QApplication.translate("menu_from_project", "&Layers menu from project", None, QApplication.UnicodeUTF8), self.act_aeag_menu_config)
        self.iface.removePluginMenu(QApplication.translate("menu_from_project", "&Layers menu from project", None, QApplication.UnicodeUTF8), self.act_aeag_menu_help)
        QObject.disconnect(self.act_aeag_menu_config, SIGNAL("triggered()"),  self.do_aeag_menu_config)
        QObject.disconnect(self.act_aeag_menu_help, SIGNAL("triggered()"),  self.do_help)

        #menuBar = self.iface.editMenu().parentWidget()

        self.store()

    def do_aeag_menu_config(self):
        dlg = menu_conf_dlg(self.iface.mainWindow(), self)
        dlg.setModal(True)
        
        dlg.show()
        result = dlg.exec_()
        del dlg
        
        if result != 0:
            self.initMenus()

    # run method that performs all the real work
    def do_aeag_menu(self, filename, who):
        #QMessageBox.information(None, "Cancel", str(filename)+str(who))
        
        # read QGis project
        xml = file(unicode(filename)).read()
        doc = QtXml.QDomDocument()
        doc.setContent(xml)
        
        layers = doc.elementsByTagName("maplayer")
        i=0
        while i<layers.count():
            node = layers.at(i)
            idNode = node.namedItem("id")
            if idNode != None:
                id = idNode.firstChild().toText().data()
                # layer founds
                if id == who:
                    # give it a new id (for multiple import)
                    import uuid
                    newLayerId = QUuid.createUuid ().toString()
                    idNode.firstChild().toText().setData(newLayerId)
                    QgsProject.instance().read(node)
                    break

            i=i+1        

    def do_help(self):
        try:
            #body = file(self.path+"/aeag.html").read()
            self.hdialog = QDialog()
            self.hdialog.setModal(True)
            self.hdialog.ui = Ui_browser()
            self.hdialog.ui.setupUi(self.hdialog)
            
            if os.path.isfile(self.path+"/help_"+self.myLocale+".html"):
                self.hdialog.ui.helpContent.setUrl(QUrl(self.path+"/help_"+self.myLocale+".html"))
            else:
                self.hdialog.ui.helpContent.setUrl(QUrl(self.path+"/help.html"))

            self.hdialog.ui.helpContent.page().setLinkDelegationPolicy(QtWebKit.QWebPage.DelegateExternalLinks) # Handle link clicks by yourself
            QObject.connect(self.hdialog.ui.helpContent, SIGNAL("linkClicked(QUrl)"), self.doLink)
            
            self.hdialog.ui.helpContent.page().currentFrame().setScrollBarPolicy(Qt.Vertical, Qt.ScrollBarAlwaysOn)
            
            #hdialog.ui.helpContent.setHtml(body)
            self.hdialog.show()
            result = self.hdialog.exec_()
            del self.hdialog
        except:
            pass
        
    def doLink( self, url ):
        if url.host() == "" :
            self.hdialog.ui.helpContent.page().currentFrame().load(url)
            #self.hdialog.ui.helpContent.page().triggerAction(QtWebKit.QWebPage.OpenLink)
            #return False #self.hdialog.ui.helpContent.setUrl(url)
        else:
            QDesktopServices.openUrl( url )
